#include "stdafx.h"
#include "cSoundPlayer.h"

// cSoundPlayer  //////////////////////////////////////////
cSoundPlayer::cSoundPlayer(){

}

cSoundPlayer::~cSoundPlayer(){

}

bool cSoundPlayer::Open(){
	int WaveOutDeviceCount;

	WaveOutDeviceCount = waveOutGetNumDevs();
	if(WaveOutDeviceCount > 0){
		if(MMSYSERR_NOERROR != waveOutGetDevCaps(0, &m_WaveOutCaps, sizeof(m_WaveOutCaps)))
			return false;
	}
	
	return true;
}

void cSoundPlayer::Close(){

}

// cSound  ////////////////////////////////////

cSound::cSound(){
	m_hWaveOut = 0;
	m_pWaveData = 0;

}

cSound::~cSound(){
	if(m_hWaveOut)
		Destroy();
}

bool cSound::Create(int Length, int Channels, int BlockAlign, int SamplesPerSec, int BitsPerSample){
	WAVEFORMATEX WaveFormat;
	
	WaveFormat.wFormatTag = WAVE_FORMAT_PCM;	// PCM  
	WaveFormat.nChannels = Channels;			// 0-mono, 1-stereo
	WaveFormat.nSamplesPerSec = SamplesPerSec;	 
	WaveFormat.wBitsPerSample = BitsPerSample;			
	WaveFormat.nBlockAlign = Channels * BitsPerSample / 8;
	WaveFormat.nAvgBytesPerSec = SamplesPerSec * WaveFormat.nBlockAlign;
	WaveFormat.cbSize = 0;


	if(MMSYSERR_NOERROR != waveOutOpen(&m_hWaveOut, 
		WAVE_MAPPER, // Default Device ID
		&WaveFormat,
		0, 0, CALLBACK_NULL))
		return false;

	return true;
}


void cSound::Destroy(){
	if(m_hWaveOut){
		waveOutReset(m_hWaveOut);
		if(m_pWaveData){
			waveOutUnprepareHeader(m_hWaveOut, &m_WaveHDR, sizeof(WAVEHDR));
			delete [] m_pWaveData;
		}
		m_pWaveData = 0;

		waveOutClose(m_hWaveOut);
		m_hWaveOut = 0;
	}
}


void cSound::Start(){
	waveOutWrite(m_hWaveOut, &m_WaveHDR, sizeof(m_WaveHDR));
}

void cSound::Stop(){
	waveOutReset(m_hWaveOut);
}

void cSound::Pause(){
	waveOutPause(m_hWaveOut);
}

void cSound::Restart(){
	waveOutReset(m_hWaveOut);
	Start();
}

typedef DWORD           FOURCC;     // 4 ڵ

typedef struct _MMCKINFO{
	FOURCC          ckid;           // ûũ ID
	DWORD           cksize;         // ûũ ũ
	FOURCC          fccType;        // ûũ  Ÿ
	DWORD           dwDataOffset;   // Ͱ ִ  ġ
	DWORD           dwFlags;        //  ÷
} MMCKINFO, *PMMCKINFO, NEAR *NPMMCKINFO, FAR *LPMMCKINFO;


bool cSound::Create(const char * pFileName){
	FILE *pFile;
	MMCKINFO Chunk;
	WAVEFORMATEX wfmtx;
	char * pData;
	unsigned int DataSize;
	unsigned int ReadSize;
	bool FormatOk = false, DataOk = false;

	//   б  
	pFile = fopen(pFileName, "rb"); 
	if(pFile == NULL)
		return false;

	//   о´.
	if(fread(&Chunk, 1, 12, pFile) < 12){
		// RIFF wave  ƴϴ.
		fclose(pFile);
		return false;
	}

	if(Chunk.ckid != mmioFOURCC('R', 'I', 'F', 'F') ||
		Chunk.fccType != mmioFOURCC('W', 'A', 'V', 'E')){
		// RIFF WAVE  ƴϴ.
		fclose(pFile);
		return false;
	}

	// ûũ о.
	for(;fread(&Chunk, 1, 8, pFile) == 8;){
		if(Chunk.ckid == mmioFOURCC('f', 'm', 't', ' ')){
			// ̺  д´.
			ReadSize = min(sizeof(wfmtx), Chunk.cksize);
			if(fread(&wfmtx, ReadSize, 1, pFile) < 1){
				//  б 
				fclose(pFile);
				return false;
			}
			FormatOk = true;
			if(ReadSize < Chunk.cksize)
				fseek(pFile, Chunk.cksize - ReadSize, SEEK_CUR);
		}else if(Chunk.ckid == mmioFOURCC('d', 'a', 't', 'a')){
			DataSize = Chunk.cksize;
			pData = new char[DataSize];
			if(fread(pData, 1, Chunk.cksize, pFile) < DataSize){
				delete [] pData;
				fclose(pFile);
				return false;
			}
			DataOk = true;

		}else{
			// ûũ Ѿ.
			fseek(pFile, Chunk.cksize, SEEK_CUR);
		}
	}

	fclose(pFile);

	if(!DataOk)
		return false;

	if(!FormatOk){
		delete [] pData;
		return false;
	}
	if(Create(DataSize, wfmtx.nChannels, wfmtx.nBlockAlign, wfmtx.nSamplesPerSec, wfmtx.wBitsPerSample)){
		LoadWave(pData, DataSize);
	}
	delete [] pData;

	return true;
}


bool cSound::LoadWave(const char * pData, int Len){
	m_pWaveData = new char[Len];
	memcpy(m_pWaveData, pData, Len);
	m_WaveDataLength = Len;

	m_WaveHDR.lpData = (LPSTR)m_pWaveData;
	m_WaveHDR.dwBufferLength = m_WaveDataLength;
	m_WaveHDR.dwBytesRecorded = 0;
	m_WaveHDR.dwUser = 0;
	m_WaveHDR.dwFlags = 0;
	m_WaveHDR.dwLoops = 0;
	m_WaveHDR.lpNext = 0;
	m_WaveHDR.reserved = 0;

	if(MMSYSERR_NOERROR != waveOutPrepareHeader(m_hWaveOut, &m_WaveHDR, sizeof(WAVEHDR)))
		return false;

	return true;

}
