#include "StdAfx.h"
#include "cPlayer.h"
#include "ChatProtocol.h"
#include "cChatSvrApp.h"



bool cPlayer::Create(SOCKET Socket, cChatSvrApp *pApp){
	if(!m_RPC.CreateFromSocket(Socket))
		return false;
	
	m_pApp = pApp;
	strcpy(m_Name, "Unknown");
	return true;
}


void cPlayer::Destroy() {
	m_RPC.Destroy();
}


bool cPlayer::Update() {

	cRPCPacket		*pPacket;
	bool			Continue = true;

	while(Continue){
		pPacket = m_RPC.PeekPacket();
		if(pPacket){
			pPacket = m_RPC.GetPacket();
			if(pPacket->GetObjectID() == RPCO_RPC){
				switch(pPacket->GetMethodID()){
				case RPCM_Close:
					m_RPC.DestroyPacket(pPacket);
					return false;
				}
			}else if(pPacket->GetObjectID() == RPCO_System){
				if(!OnClientMessage(pPacket->GetObjectID(), pPacket->GetMethodID(), pPacket)){
					m_RPC.DestroyPacket(pPacket);
					return false;
				}
			}else{
				if(!OnClientMessage(pPacket->GetObjectID(), pPacket->GetMethodID(), pPacket)){
					m_RPC.DestroyPacket(pPacket);
					return false;
				}
			}
			m_RPC.DestroyPacket(pPacket);
		}else{
			Continue = false;
		}
	}
	return true;
}


bool cPlayer::OnClientMessage(unsigned char ObjectID, unsigned char MethodID, cRPCPacket *pPacket) {
	
	switch(MethodID){
	case CSM_Login:
		{
			printf("CSM_Login\n");
			char PlayerName[20];
			char Success = 1;
			CSM_Login_Stub (*pPacket, PlayerName);
			strcpy (m_Name, PlayerName);
			cRPCPacket NewPacket;
			NewPacket.SetHeader (RPCO_Owner, CCM_LoginResult);
			CCM_LoginResult_Stub ( NewPacket, Success);
			GetRPC()->SendPacket ( &NewPacket );
		}
		break;

	case CSM_SetChatMode:
		{
			printf("CSM_ChatMode\n");
			cRPCPacket NewPacket;
			NewPacket.SetHeader (RPCO_Owner, CCM_SetName);
			CCM_SetName_Stub( NewPacket, m_Name);
			GetRPC()->SendPacket ( &NewPacket );
			cRPCPacket NewPacket2;
			NewPacket2.SetHeader (RPCO_Owner, CCM_NewPlayer);
			CCM_NewPlayer_Stub( NewPacket2, m_Name);
			GetApp()->BroadcastPacket( &NewPacket2, this);

		}
		break;
	case CSM_AddChat:
		{
			printf("CSM_AddChat : [%s] ",m_Name);
			char Chat[256];
			CSM_AddChat_Stub(*pPacket, Chat);
			printf( "%s\n", Chat);
			AddChat(Chat);

		}
		break;
	case CSM_LogOut:
		{
//			GetRPC()->SendSimplePacket (RPCO_Owner, CCM_LogOutAccept);
			return false;
		}
		break;

	}
	return true;
}


void cPlayer::Dump(){
	printf("Player(%X) Dump\n", (int)this);
	printf("\tName		= %s\n", m_Name);
}



bool cPlayer::AddChat(const char *pChat){
	cRPCPacket	Packet;

	Packet.SetHeader(RPCO_Owner, CCM_AddChat);
	CCM_AddChat_Stub(Packet, m_Name, (char*)pChat);

	GetApp()->BroadcastPacket(&Packet, this);

	return true;
}