#include "StdAfx.h"
#include "ROGLobby.h"
#include "ROGLobbyProtocol.h"
#include "ROGServer.h"

bool cROGLobbyGamePlayer::OnClientMessage(unsigned char ObjectID, unsigned char MethodID, cRPCPacket *pPacket){
	// 
	switch(MethodID){
	case ROGLSM_GetPlayerList:
		{
			printf("ROGLSM_GetPlayerList\n");
			GetPlayerList();
		}
		break;
	case ROGLSM_GetRoomList:
		{
			printf("ROGLSM_GetRoomList\n");
			GetRoomList();
		}
		break;
	case ROGLSM_CreateRoom:
		{
			printf("ROGLSM_CreateRoom\n");
			short PlayerCountLimit;
			ROGLSM_CreateRoom_Stub(*pPacket, PlayerCountLimit);
			CreateRoom(PlayerCountLimit);
		}
		break;
	case ROGLSM_JoinRoom:
		{
			printf("ROGLSM_JoinRoom\n");
			short RoomID;
			ROGLSM_JoinRoom_Stub(*pPacket, RoomID);
			JoinRoom(RoomID);
		}
		break;
	case ROGLSM_AddChat:
		{
			printf("ROGLSM_AddChat\n");
			char Chat[256];
			ROGLSM_AddChat_Stub(*pPacket, Chat);
			AddChat(Chat);

		}
		break;
	case ROGLSM_WisperChat:
		{
			printf("ROGLSM_WisperChat\n");
			char to[256];
			char Chat[256];
			ROGLSM_WisperChat_Stub(*pPacket, to, Chat);
			WisperChat(to, Chat);
		}
		break;

	case ROGLSM_SetChipNumber:
		{
			printf("ROGLSM_SetChipNumber\n");
			int  nChipNumber;
			ROGLSM_SetChipNumber_Stub(*pPacket, nChipNumber);
			SetChipNumber(nChipNumber);
		
		}
		break;


	case ROGLSM_SetMoneyNumber:
		{
			printf("ROGLSM_SetMoneyNumber\n");
			int  nMoneyNumber;
			ROGLSM_SetMoneyNumber_Stub(*pPacket, nMoneyNumber);
			SetMoneyNumber(nMoneyNumber);
		
		}
		break;

	case ROGLSM_GetMoney:
		{
			printf("ROGLSM_GetMoney\n");
			SendMoney();
		}
		break;

	case ROGLSM_GetName:
		{
			printf("ROGLSM_GetName\n");
			cRPCPacket Packet;
			Packet.SetHeader (RPCO_Owner, ROGLCM_SetName);
			Packet + pPlayer->GetName();
			SendPacket (&Packet);
		}
		break;

	default:
		ASSERT(FALSE);
		break;
	};

	return true;
}

cROGLobby* cROGLobbyGamePlayer::GetLobby(){
	return (cROGLobby*)pPlayer->GetRoom();
}

bool cROGLobbyGamePlayer::GetPlayerList(){
	cRPCPacket		Packet;
	sROGPlayerData	*pPlayerData;
	unsigned short	PlayerCount;

	PlayerCount = pApp->GetPlayerListData(pPlayerData);

	Packet.SetHeader(ROGLCO_PlayerView, ROGLCM_SetPlayerList);
	ROGLCM_SetPlayerList_Stub(Packet, PlayerCount, pPlayerData);

	if(PlayerCount > 0)
		delete [] pPlayerData;

	return SendPacket(&Packet);
	
}


bool cROGLobbyGamePlayer::GetRoomList(){
	cRPCPacket		Packet;
	sROGRoomData	*pRoomData;
	short			RoomCount;


	RoomCount = pApp->GetRoomListData(pRoomData);

	Packet.SetHeader(ROGLCO_RoomView, ROGLCM_SetRoomList);
	ROGLCM_SetRoomList_Stub(Packet, RoomCount, pRoomData);

	if(RoomCount > 0)
		delete [] pRoomData;

	return SendPacket(&Packet);
	
}

bool cROGLobbyGamePlayer::CreateRoom(short PlayerCountLimit){
	// ο   װ 
	short NewRoomID;

	short			Result;
	cRPC		*pRPC	= &pPlayer->RPC;
	cRPCPacket		Packet(4 + sizeof(short));

	//Ĩ  ӹ  .
	if(pPlayer->GetChip() <= 0){
		Result			= -1;
		Packet.SetHeader(ROGLCO_RoomView, ROGLCM_CreateRoomResult);
		ROGLCM_CreateRoomResult_Stub(Packet, Result);
		pRPC->SendPacket(&Packet);
		
		return false;
	}


	NewRoomID = pApp->CreateRoom(PlayerCountLimit);
	if(NewRoomID < 0)
		return false;
	return JoinRoom(NewRoomID);

}

bool cROGLobbyGamePlayer::JoinRoom(short RoomID){
	//  濡 


	short			Result;
	cRPC		*pRPC	= &pPlayer->RPC;
	cRPCPacket		Packet(4 + sizeof(short));

	//Ĩ  ӹ濡 鿩 ʴ´.
	if(pPlayer->GetChip() <= 0){
		Result			= -1;
		Packet.SetHeader(ROGLCO_RoomView, ROGLCM_JoinRoomResult);
		ROGLCM_JoinRoomResult_Stub(Packet, Result);
		pRPC->SendPacket(&Packet);
		
		return false;
	}

	if(!pApp->JoinRoom(RoomID, pPlayer))
		return false;
	
	

	Result			= 1;
	Packet.SetHeader(ROGLCO_RoomView, ROGLCM_JoinRoomResult);
	ROGLCM_JoinRoomResult_Stub(Packet, Result);
//	Sleep(1000);
	pRPC->SendPacket(&Packet);
	return true;
}

bool cROGLobbyGamePlayer::AddChat(const char *pChat){
	cRPCPacket	Packet;

	Packet.SetHeader(ROGLCO_Lobby, ROGLCM_AddChat);
	ROGLCM_AddChat_Stub(Packet, (char*)pPlayer->GetName(), (char*)pChat);

	BroadcastPacket(&Packet, this);

	return true;
}

bool cROGLobbyGamePlayer::WisperChat(const char *pTo, const char *pChat){
	cRPCPacket Packet;
	char str[100];

	Packet.SetHeader(ROGLCO_Lobby, ROGLCM_AddChat);
	sprintf(str, "[%s]", pPlayer->GetName());
	ROGLCM_AddChat_Stub(Packet, (char*)str, (char*)pChat);
	
	cROGPlayer *pTargetPlayer;
	pTargetPlayer = pApp->FindPlayer(pTo);
	if(pTargetPlayer && pTargetPlayer != pPlayer){
		pTargetPlayer->GetRPC()->SendPacket(&Packet);
	}

	
	return true;
}

// cROGLobby /////////////////////////////////////////////////////////////////////////////////////////////
cROGLobby::cROGLobby(){

}

cROGLobby::~cROGLobby(){

}


bool cROGLobby::Create(short NewRoomID, int NewPlayerCountLimit){
	if(!cROGRoom::Create(NewRoomID, NewPlayerCountLimit))
		return false;
	return true;
}

void cROGLobby::Destroy(){
	cROGRoom::Destroy();
}

bool cROGLobby::AddPlayer(cROGPlayer *pNewPlayer){
	if(!cROGRoom::AddPlayer(pNewPlayer))
		return false;

	cROGLobbyGamePlayer			*pLobbyPlayer;
	pLobbyPlayer				= new cROGLobbyGamePlayer;
	pNewPlayer->SetGamePlayer(pLobbyPlayer);

	return true;

}

void cROGLobby::RemovePlayer(cROGPlayer *pPlayer){
	//  ü .
	delete pPlayer->GetGamePlayer();
	pPlayer->SetGamePlayer(0);

	cROGRoom::RemovePlayer(pPlayer);


}

bool cROGLobbyGamePlayer::SendMoney(){
	cRPCPacket		Packet;

	int nMoney, nChip;

	nMoney = pPlayer->GetMoney();
 	nChip = pPlayer->GetChip();
	
	Packet.SetHeader(ROGLCO_MoneyView, ROGLCM_SetMoney);
	ROGLCM_SetMoney_Stub(Packet, nMoney, nChip);

	
	return SendPacket(&Packet);
	
}

bool cROGLobbyGamePlayer::SetChipNumber(int _nChipNumber){
	cRPCPacket		Packet;
	int nMoney;

	//Ĩ شϴ  ش.
	pPlayer->SetChipNumber(_nChipNumber);
	nMoney = pPlayer->GetMoney();
	nMoney -= _nChipNumber;
	pPlayer->SetMoney(nMoney);
	//SetMoney()  Ŀ ׻ db   ؾѴ.
	pPlayer->SaveIntoDB(&pApp->DBRecord);

	Packet.SetHeader(ROGLCO_MoneyView, ROGLCM_SetMoney);
	ROGLCM_SetMoney_Stub(Packet, nMoney, _nChipNumber);

	
	return SendPacket(&Packet);

	
	
	
}


bool cROGLobbyGamePlayer::SetMoneyNumber(int _nMoneyNumber){
	cRPCPacket		Packet;
	int nChip;
	int nMoney;


	nChip = pPlayer->GetChip();
	
	if(_nMoneyNumber > nChip)
		_nMoneyNumber = nChip;

	nChip -= _nMoneyNumber;

	//Ĩ  ȯ ش.
	pPlayer->SetChipNumber(nChip);

	nMoney = pPlayer->GetMoney();
	nMoney += _nMoneyNumber;
	pPlayer->SetMoney(nMoney);
	//SetMoney()  Ŀ ׻ db   ؾѴ.
	pPlayer->SaveIntoDB(&pApp->DBRecord);

	Packet.SetHeader(ROGLCO_MoneyView, ROGLCM_SetMoney);
	ROGLCM_SetMoney_Stub(Packet, nMoney, nChip);

	
	return SendPacket(&Packet);

	
	
	
}







