#include "StdAfx.h"
#include "ROGPlayer.h"
#include "ROGRoom.h"
#include "ROGLobbyProtocol.h"
#include "ROGServer.h"

// cROGGamePlayer Ŭ 
bool cROGGamePlayer::SendPacket(cRPCPacket *pPacket){
	return pPlayer->GetRPC()->SendPacket(pPacket);
}

bool cROGGamePlayer::BroadcastPacket(cRPCPacket *pPacket, cROGGamePlayer *pPlayerExcept){
	return pPlayer->GetRoom()->BroadcastPacket(pPacket, pPlayerExcept);
}

// cROGPlayer Ŭ 
cROGPlayer::cROGPlayer(){
	pRoom = 0;
}

cROGPlayer::~cROGPlayer(){
}

void cROGPlayer::Dump(){
	printf("Player(%X) Dump\n", (int)this);
	printf("\tName		= %s\n", pName);
}

bool cROGPlayer::Create(SOCKET Socket){
	if(!RPC.CreateFromSocket(Socket))
		return false;

	pGamePlayer = 0;
	strcpy(pName, "Unknown");
	Logged = false;
	return true;
}

void cROGPlayer::Destroy(){
	RPC.Destroy();
	if(Logged)
		Logout();
}

void cROGPlayer::SetGamePlayer(cROGGamePlayer *pNewGamePlayer){
	pGamePlayer					= pNewGamePlayer;
	if(pGamePlayer)
		pGamePlayer->pPlayer	= this;

}

cROGGamePlayer *cROGPlayer::GetGamePlayer(){
	return pGamePlayer;
}

cRPC* cROGPlayer::GetRPC(){
	return &RPC;
}

void cROGPlayer::SetRoom(cROGRoom *pNewRoom){
	pRoom = pNewRoom;
	if(pNewRoom)
		BroadcastPlayerUpdate();
}

cROGRoom* cROGPlayer::GetRoom(){
	return pRoom;
}

bool cROGPlayer::Update(){
	cRPCPacket		*pPacket;
	bool			Continue = true;

	while(Continue){
		pPacket = RPC.PeekPacket();
		if(pPacket){
			pPacket = RPC.GetPacket();
			if(pPacket->GetObjectID() == RPCO_RPC){
				switch(pPacket->GetMethodID()){
				case RPCM_Close:
					RPC.DestroyPacket(pPacket);
					return false;
				}
			}else if(pPacket->GetObjectID() == RPCO_System){
				if(!OnClientMessage(pPacket->GetObjectID(), pPacket->GetMethodID(), pPacket)){
					RPC.DestroyPacket(pPacket);
					return false;
				}
			}else{
				if(pGamePlayer){
					if(!pGamePlayer->OnClientMessage(pPacket->GetObjectID(), pPacket->GetMethodID(), pPacket)){
						RPC.DestroyPacket(pPacket);
						return false;
					}
				}else{
					if(!OnClientMessage(pPacket->GetObjectID(), pPacket->GetMethodID(), pPacket)){
						RPC.DestroyPacket(pPacket);
						return false;
					}
				}
			}
			RPC.DestroyPacket(pPacket);
		}else{
			Continue = false;
		}
	}
	return true;

}


bool cROGPlayer::OnClientMessage(unsigned char ObjectID, unsigned char MethodID, cRPCPacket *pPacket){
	switch(MethodID){
	case ROGLSM_Login:
		{
			char PlayerName[20];
			char Password[20];
			ROGLSM_Login_Stub(*pPacket, PlayerName, Password);

			char Success = Login(PlayerName, Password);

			cRPCPacket NewPacket;
			NewPacket.SetHeader(RPCO_Owner, ROGLCM_LoginResult);
			ROGLCM_LoginResult_Stub(NewPacket, Success);
			GetRPC()->SendPacket(&NewPacket);
		}
		break;

	case ROGLSM_SetLobbyMode:
		pApp->LeaveRoom(this);
		break;

	case ROGLSM_CreateNewPlayer:
		{
			char PlayerName[20];
			char Password[20];
			char email[30];
			char Gender;
			int BirthYear;

			ROGLSM_CreateNewPlayer_Stub(*pPacket, PlayerName, Password, email, Gender, BirthYear);

			char Result = CreateNewPlayer(PlayerName, Password, email, Gender, BirthYear);

			cRPCPacket NewPacket;
			NewPacket.SetHeader(RPCO_Owner, ROGLCM_CreateNewPlayerResult);
			ROGLCM_CreateNewPlayerResult_Stub(NewPacket, Result);
			GetRPC()->SendPacket(&NewPacket);

		}
		break;

	default:
		ASSERT(FALSE);
		break;
	};

	return true;
}


void cROGPlayer::BroadcastPlayerUpdate(){
	// ٸ ڵ鿡 ڰ ° ˸.
	cRPCPacket Packet;
	sROGPlayerData PlayerData;
	Packet.SetHeader(ROGLCO_PlayerView, ROGLCM_UpdatePlayerList);
	strcpy(PlayerData.Name, GetName());
	PlayerData.RoomID = GetRoom()->GetRoomID();
	PlayerData.Money = 0;
	ROGLCM_UpdatePlayerList_Stub(Packet, &PlayerData);
	pApp->Lobby.BroadcastPacket(&Packet);
}


bool cROGPlayer::Login(const char *PlayerName, const char *Password){
	strcpy(pName, PlayerName);
	strcpy(pPassword, Password);
	if(!LoadFromDB(&pApp->DBRecord))
		return false;

	int nBackUpMoney = GetBackUpMoney();

	int nMoney;

	//nBackUpMoney  0 Ŭ 쿣  ῡ   ̹Ƿ 
	//Money BackUpMoney ش.
	//߿  ؼ  Ȯȯ DB  ĸ ߰ڴ. ߿,,,
	if(nBackUpMoney > 0){
		nMoney = nBackUpMoney;
	}else
		nMoney = GetMoney();

		
	// Ŀؼ  츦 ؼ Ӵ  صд.
	BackUpMoneyIntoDB(nMoney);

	

	//Ʈ   Ĩ .
	nChip = nMoney;
	nMoney = 0;

	
	SetMoney(nMoney);



	//Ĩ  ٲ  Ѵ.
	SaveIntoDB(&pApp->DBRecord);


	Logged = true;

	pApp->LeaveRoom(this);
	// ٸ ڵ鿡 ڰ ° ˸.
	cRPCPacket Packet;
	sROGPlayerData PlayerData;
	Packet.SetHeader(ROGLCO_PlayerView, ROGLCM_AddPlayerList);
	strcpy(PlayerData.Name, GetName());
	PlayerData.RoomID = GetRoom()->GetRoomID();
	PlayerData.Money = 0;
	ROGLCM_AddPlayerList_Stub(Packet, &PlayerData);
	pApp->Lobby.BroadcastPacket(&Packet, this);

	return true;
}


void cROGPlayer::Logout(){
	//Ĩ ٽ  ٲ۴.
	int nMoney = GetMoney();
	SetMoney(nMoney + nChip);
	
	//   쿡 backupmoney 0 Ѵ. 
	BackUpMoneyIntoDB(0);

	SaveIntoDB(&pApp->DBRecord);
	Logged = false;
}



int cROGPlayer::CreateNewPlayer(const char *NewPlayerName, const char *NewPassword, 
								 const char *email, char Gender, int BirthYear){
	// Ʈ  
	cDBRecord *pRecord = &pApp->DBRecord;

	if(pRecord->Query("select count(*) from user where id='%s'", NewPlayerName)){
		if(pRecord->Fetch()){
			int PlayerCount;
			*pRecord >> PlayerCount;
			if(PlayerCount > 0)
				return 2;
		}
	}

	if(pRecord->Query("insert into user (id,pw,money,email,gender,BirthYear,backupmoney) values('%s', password('%s'), 10000, '%s', %d, %d, 10000)", 
		NewPlayerName, NewPassword, email, Gender, BirthYear))
		return 0;


	return 1;
}

//  ̽ ó
bool cROGPlayer::LoadFromDB(cDBRecord *pRecord){
	if(!pRecord->Query("select id from user where id='%s' and pw=password('%s')", pName, pPassword))
		return false;

	if(!pRecord->Fetch())
		return false;

	
	return true;
}

bool cROGPlayer::SaveIntoDB(cDBRecord *pRecord){
	return true;
}

bool cROGPlayer::SetMoney(int _Money){
	cDBRecord *pRecord = &pApp->DBRecord;

	if(!pRecord->Query("update user set money=%d where id='%s'", _Money, pName)){
		ASSERT(!"Save DB Error");
		return false;
	}

	return true;
}

int cROGPlayer::GetMoney(){ 
	int tmpMoney;

	cDBRecord *pRecord = &pApp->DBRecord;

	if(!pRecord->Query("select money from user where id='%s' and pw=password('%s')", pName, pPassword))
		return false;
	if(!pRecord->Fetch())
		return false;

	*pRecord >> tmpMoney;
	pRecord->EndFetch();

	return tmpMoney;
}

bool cROGPlayer::BackUpMoneyIntoDB(int _nMoney){

	cDBRecord *pRecord = &pApp->DBRecord;

	if(!pRecord->Query("update user set backupmoney=%d where id='%s'", _nMoney, pName)){
		ASSERT(!"Save BackUp Money Error");
		return false;
	}

	return true;
}

int cROGPlayer::GetBackUpMoney(){ 

	int tmpMoney;

	// Ʈ  
	cDBRecord *pRecord = &pApp->DBRecord;

	if(!pRecord->Query("select backupmoney from user where id='%s' and pw=password('%s')", pName, pPassword))
		return false;
	if(!pRecord->Fetch())
		return false;

	*pRecord >> tmpMoney;
	pRecord->EndFetch();

	return tmpMoney;
}









