#include "StdAfx.h"
#include "ROGServerApp.h"
#include "ROGPlayer.h"
#include "ROGRoom.h"
#include "ROGLobby.h"
#include "ROGLobbyProtocol.h"

extern cROGRoom *ROGCreateRoom();
cROGServerApp *pApp;

cROGServerApp::cROGServerApp(){
	pApp = this;

}

cROGServerApp::~cROGServerApp(){
	pApp = NULL;
}


bool cROGServerApp::OpenDB(){

	printf("cROGServerApp::OpenDB()\n");

	if(!DBCon.Connect("gamezone.renodigm.com", "gamezone", "gamezone", "gamezone", 0)){
		printf("\tfail.\n");
		return false;
	}

    DBRecord.Open(&DBCon);
	printf("\tOK.\n");

	return true;
}

void cROGServerApp::CloseDB(){
	printf("cROGServerApp::CloseDB()\n");

	DBRecord.Close();

	DBCon.Close();

	printf("\tOK.\n");
}


bool cROGServerApp::OpenRPC(){
	printf("cROGServerApp::OpenRPC()\n");

#ifdef WIN32
	WSADATA wsaData;
	if(WSAStartup(MAKEWORD( 2, 2 ), &wsaData ) != 0){
		printf("\tWSAStartup() fail.\n");
		return false;
	}

#endif

	if(!RPC.CreateServer(NULL, GetServerPort(), 10)){
		printf("\tRPC.CreateServer() Fail.\n");
		return false;
	}
	printf("\tOK.\n");
	return true;
}

void cROGServerApp::CloseRPC(){
	printf("cROGServerApp::CloseRPC()\n");

	RPC.Destroy();

	printf("\tOK.\n");
}

bool cROGServerApp::AcceptNewPlayer(SOCKET Socket){
	cROGPlayer *pNewPlayer;

	printf("cROGServerApp::AcceptNewPlayer(%d)\n", Socket);

	pNewPlayer = new cROGPlayer;
	if(!pNewPlayer->Create(Socket)){
		shutdown(Socket, 0);
		closesocket(Socket);
		printf("\tpNewPlayer->Create() Failed.\n");
	}
//	PlayerList.AddTail(pNewPlayer);
	printf("\tOK.\n");

	// ӵ  ϴ  ִ.α ϰԵǸ κ  ȴ.
	PlayerList.AddTail(pNewPlayer);

	return true;
}

void cROGServerApp::UpdateRooms(){
	cList<cROGRoom *>::cCursor	csr;
	cROGRoom					*pRoom;

	Lobby.Update();

	for(csr = RoomList.Head(); csr != 0;){
		pRoom = *csr;
		if(pRoom->Update()){
			csr ++;

		}else{
			//  ۾
			printf("cROGServerApp::UpdateRoooms()\n");
			printf("\tDestroying Room\n");
			pRoom->Dump();
			FreeRoomIDList.AddTail(pRoom->GetRoomID());
			pRoom->Destroy();
			delete pRoom;

			csr = RoomList.Delete(csr);
			printf("\tOK.\n");

		}

	}

}

void cROGServerApp::UpdatePlayers(){
	cList<cROGPlayer *>::cCursor	csr;
	cROGPlayer						*pPlayer;

	for(csr = PlayerList.Head(); csr != 0;){
		pPlayer = *csr;
		if(pPlayer->Update()){
			csr ++;

		}else{
			// ÷̾ ۾
			csr = PlayerList.Delete(csr);

			printf("cROGServerApp::UpdatePlayers() Destroying Player\n");
//			ASSERT(pPlayer->GetRoom());
			// ٸ ڵ鿡 ڰ  ˸.
			cRPCPacket Packet;
			Packet.SetHeader(ROGLCO_PlayerView, ROGLCM_RemovePlayerList);
			ROGLCM_RemovePlayerList_Stub(Packet, (char *)pPlayer->GetName());
			Lobby.BroadcastPacket(&Packet, pPlayer->GetGamePlayer());


			if(pPlayer->GetRoom())
				pPlayer->GetRoom()->RemovePlayer(pPlayer);
			pPlayer->Dump();
			pPlayer->Destroy();
			delete pPlayer;

		}

	}
}


int cROGServerApp::Main(int argc, char **argv){
	printf("cROGServerApp::Main() Openning...\n");

	if(!OpenDB()){ //  ̽ ʱȭ Ѵ.
		return -1;
	}

	if(!OpenRPC()){ //  ν ȣ ʱȭ Ѵ.
		return -1;
	}

	if(Lobby.Create(0, 0)){
		printf("cROGServerApp::Main()::Lobby.Create() OK.\n");
	}else{
		printf("cROGServerApp::Main()::Lobby.Create() Fail.\n");
		return -1;
	}
	int i;
	// Free Room ID Ҵ
	for(i = 1; i <= 100; i ++)
		FreeRoomIDList.AddTail((short)i);

	printf("cROGServerApp::Main() Open OK.\n");

	// Main Loop
	cRPCPacket *pPacket;
	bool Continue = true;

	while(Continue){

		pPacket = RPC.PeekPacket();
		if(pPacket){
			pPacket = RPC.GetPacket();
			ASSERT(pPacket->GetObjectID() == RPCO_RPC);

			switch(pPacket->GetMethodID()){
			case RPCM_Accept:
				{	// ο ڰ 
					SOCKET Socket;
					cRPC::AcceptStub(*pPacket, Socket);
					AcceptNewPlayer(Socket);
				}
				break;

			case RPCM_Close:
				Continue = false;
				break;
			}
			RPC.DestroyPacket(pPacket);

		}else{
			UpdateRooms();		//   Ʈ
			UpdatePlayers();	// κ ִ ڵ Ʈ
		}

	}

	// End of Main loop



	printf("cROGServerApp::Main() Closing...\n");

	FreeRoomIDList.RemoveAll();
	CloseRPC();
	CloseDB();

	printf("cROGServerApp::Main() OK\n");

	return 0;
}

short cROGServerApp::GetPlayerListData(sROGPlayerData * & pPlayerData){

	cROGPlayer		*pPlayer;
	unsigned short	PlayerCount;
	int				i;
	cList<cROGPlayer *>::cCursor csr;

	PlayerCount = PlayerList.GetCount();
	if(PlayerCount > 0){
		pPlayerData = new sROGPlayerData[PlayerCount];
		csr = PlayerList.Head();
		for(i = 0; i < PlayerCount; i ++){
			pPlayer = *csr;
			strcpy(pPlayerData[i].Name, pPlayer->GetName());
			pPlayerData[i].Money = pPlayer->GetMoney();
			if(pPlayer->GetRoom())
				pPlayerData[i].RoomID = pPlayer->GetRoom()->GetRoomID();
			else
				pPlayerData[i].RoomID = 0;

			csr ++;
		}
	}
	return PlayerCount;
}

short cROGServerApp::GetRoomListData(sROGRoomData * & pRoomData){
	cROGRoom		*pRoom;
	unsigned char	RoomCount;
	int				i;
	cList<cROGRoom *>::cCursor csr;

	RoomCount = RoomList.GetCount();
	if(RoomCount > 0){
		pRoomData = new sROGRoomData[RoomCount];
		csr = RoomList.Head();
		for(i = 0; i < RoomCount; i ++){
			pRoom = *csr;
			pRoomData[i].RoomID				= (unsigned char)pRoom->GetRoomID();
			pRoomData[i].PlayerCount		= pRoom->GetPlayerCount();
			pRoomData[i].PlayerCountLimit	= pRoom->GetPlayerCountLimit();

			csr ++;
		}
	}

	return RoomCount;
}

short cROGServerApp::CreateRoom(int PlayerCountLimit){
	// ϳ   .
	printf("cROGServerApp::CreateRoom()\n");

	short NewRoomID;

	if(FreeRoomIDList.GetCount() == 0){
		printf("\tcount limit exceeded.\n");
		return -1;
	}
	NewRoomID = FreeRoomIDList.GetHead();
	FreeRoomIDList.RemoveHead();

	cROGRoom *pNewRoom;

	pNewRoom	= ROGCreateRoom();
	if(!pNewRoom->Create(NewRoomID, PlayerCountLimit)){
		FreeRoomIDList.AddTail(NewRoomID);
		delete pNewRoom;
		printf("\tCreate Failed.\n");
		return -1;
	}

	RoomList.AddTail(pNewRoom);
	
	printf("\tRoomID: %d\n", NewRoomID);
	printf("\tLimit : %d\n", PlayerCountLimit);
	printf("\tOk.\n");


	return NewRoomID;

}

bool cROGServerApp::JoinRoom(short RoomID, cROGPlayer *pPlayer){
	// شϴ ȣ .
	printf("cROGServerApp::JoinRoom()\n");

	// ȣ ã´.
	cList<cROGRoom *>::cCursor	csr;
	cROGRoom					*pRoom;

	for(csr = RoomList.Head(); csr != 0; csr ++){
		pRoom = *csr;
		if(pRoom->GetRoomID() == RoomID){
			//   ο Ѵ.
			if(pRoom->IsFull()){
				printf("\tThe room(%d) is full.\n", RoomID);
				return false;
			}

			if(pPlayer->GetRoom())
				pPlayer->GetRoom()->RemovePlayer(pPlayer);

			
			if(!pRoom->AddPlayer(pPlayer)){
				printf("\tpRoom->AddPlayer() Failed.\n");
				return false;
			}
			printf("\tRoomID: %d\n", RoomID);
			pPlayer->Dump();
			printf("\tOK.\n");

			return true;
		}
	}
	printf("\tNo room available.\n");
	return false;
}

void cROGServerApp::LeaveRoom(cROGPlayer *pPlayer){ // κ .
//	printf("cROGServerApp::LeaveRoom()\n");
	cROGRoom *pRoom;

	pRoom = pPlayer->GetRoom();

	if(pRoom == &Lobby)
		return ;

	if(pRoom)
		pRoom->RemovePlayer(pPlayer);

	Lobby.AddPlayer(pPlayer);
	//     Ŭ̾Ʈ Ѵ.
	cRPCPacket Packet;
	sROGRoomData RoomData;

	if(pRoom){
		Packet.SetHeader(ROGLCO_RoomView, ROGLCM_UpdateRoomList);
		RoomData.RoomID = pRoom->GetRoomID();
		RoomData.PlayerCount = pRoom->GetPlayerCount();
		RoomData.PlayerCountLimit = pRoom->GetPlayerCountLimit();
		ROGLCM_UpdateRoomList_Stub(Packet, &RoomData);
		Lobby.BroadcastPacket(&Packet, pPlayer);
	}


}


cROGPlayer* cROGServerApp::FindPlayer(const char *pName){
	cList<cROGPlayer *>::cCursor csr;
	cROGPlayer *pPlayer;

	for(csr = PlayerList.Head(); csr != 0; csr ++){
		pPlayer = *csr;

		if(strcmp(pPlayer->GetName(), pName) == 0){
			return pPlayer;
		}
		

	}
	return 0;
}

bool cROGServerApp::CreateNewAccount(const char *pUserID, 
				 const char *pPassword, 
				 const char Email,
				 bool IsMale,
				 int BirthYear){
	// ο  .
	if(!DBRecord.Query("insert into user values('%s', password('%s'), 0)",
		pUserID, pPassword))
		return false;
	return true;
}
/*
bool cROGServerApp::Login(cROGPlayer *pPlayer,
		const char *pUserID,
		const char *pPassword){
	// ROGS αѴ.
	pPlayer->LoadFromDB(pUserID, pPassword);
	return true;
}*/





